<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @since      1.0.0
 *
 * @package    Affiliate_Butler_Pro
 * @subpackage Affiliate_Butler_Pro/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Affiliate_Butler_Pro
 * @subpackage Affiliate_Butler_Pro/admin
 * @author     Waseem Senjer <waseem.senjer@gmail.com>
 */
class Affiliate_Butler_Pro_Admin {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $Affiliate_Butler_Pro    The ID of this plugin.
	 */
	private $Affiliate_Butler_Pro;
	protected $key = 'abutler_settings';

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;
	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $last_link_id;
	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $Affiliate_Butler_Pro       The name of this plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $Affiliate_Butler_Pro, $version ) {

		$this->Affiliate_Butler_Pro = $Affiliate_Butler_Pro;
		$this->version = $version;
		$this->last_link_id = abutlerpro_base62::encode(get_option('last_lar_link_id') + 100);

	}

	/**
	 * Register the stylesheets for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Affiliate_Butler_Pro_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Affiliate_Butler_Pro_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		wp_enqueue_style( $this->Affiliate_Butler_Pro, plugin_dir_url( __FILE__ ) . 'css/abutler-links-admin.css', array(), $this->version, 'all' );
		wp_enqueue_style( $this->Affiliate_Butler_Pro.'-jqvmap', plugin_dir_url( __FILE__ ) . 'js/jqvmap/jqvmap.css', array(), $this->version, 'all' );
		wp_enqueue_style( $this->Affiliate_Butler_Pro.'-select2', ABUTLERPRO_URL . 'admin/css/select2.min.css', array(), $this->version, 'all' );
		wp_enqueue_style( $this->Affiliate_Butler_Pro.'-popup', ABUTLERPRO_URL . 'public/css/popup.css', array(), $this->version, 'all' );
		wp_enqueue_style( $this->Affiliate_Butler_Pro.'-jbox', ABUTLERPRO_URL . 'public/css/jbox.css', array(), $this->version, 'all' );


	}

	/**
	 * Register the JavaScript for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Affiliate_Butler_Pro_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Affiliate_Butler_Pro_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		wp_enqueue_script( $this->Affiliate_Butler_Pro, plugin_dir_url( __FILE__ ) . 'js/abutler-links-admin.js', array( 'jquery' ), $this->version, false );
	 	wp_enqueue_script( $this->Affiliate_Butler_Pro.'-select2', ABUTLERPRO_URL . 'admin/js/select2.min.js', array( 'jquery' ), $this->version, false );
		wp_enqueue_script( $this->Affiliate_Butler_Pro.'-popup', ABUTLERPRO_URL . 'public/js/popup.js' , array( 'jquery' ), $this->version, false );
		wp_enqueue_script( $this->Affiliate_Butler_Pro.'-jbox', ABUTLERPRO_URL . 'public/js/jbox.min.js' , array( 'jquery' ), $this->version, false );

	}
	/**
	 * Unregister the JavaScript which possibly conflict with the plugin
	 *
	 * @since    1.1.0
	 */
	public function dequeue_conflicted_scripts(){
		if(is_admin()){
			// 1- dequeuing Thrive Themes select2 conflict.
			if(isset($_GET['post']) ){
				if(get_post_type($_GET['post']) == 'lar_link'){
					wp_dequeue_script('thrive-select2');
				}
			}
			if((isset($_GET['post_type']) && $_GET['post_type']== 'lar_link')){
				wp_dequeue_script('thrive-select2');
			}
		}
	}

	/**
	 * The plugin use this method to register the main custom post type of the links.
	 *
	 * @since    1.0.0
	 */
	public function register_links_post_type(){
		$labels = array(
			'name'               => _x( 'Auto Links', 'post type general name', 'affiliate-butler-pro' ),
			'singular_name'      => _x( 'Auto Link', 'post type singular name', 'affiliate-butler-pro' ),
			'menu_name'          => _x( 'Affiliate Butler', 'admin menu', 'affiliate-butler-pro' ),
			'name_admin_bar'     => _x( 'Auto Link', 'add new on admin bar', 'affiliate-butler-pro' ),
			'add_new'            => _x( 'Add New Auto Link', 'book', 'affiliate-butler-pro' ),
			'add_new_item'       => __( 'Add New Auto Link', 'affiliate-butler-pro' ),
			'new_item'           => __( 'New Auto Link', 'affiliate-butler-pro' ),
			'edit_item'          => __( 'Edit Auto Link', 'affiliate-butler-pro' ),
			'view_item'          => __( 'View Auto Link', 'affiliate-butler-pro' ),
			'all_items'          => __( 'All Auto Links', 'affiliate-butler-pro' ),
			'search_items'       => __( 'Search Auto Links', 'affiliate-butler-pro' ),
			'parent_item_colon'  => __( 'Parent Auto Links:', 'affiliate-butler-pro' ),
			'not_found'          => __( 'No links found.', 'affiliate-butler-pro' ),
			'not_found_in_trash' => __( 'No links found in Trash.', 'affiliate-butler-pro' )
		);

		$args = array(
			'labels'             => $labels,
			'public'             => true,
			'publicly_queryable' => false,
			'show_ui'            => true,
			'show_in_menu'       => true,
			'query_var'          => true,
			'rewrite'            => array( 'slug' => 'lar_link' ),
			'capability_type'    => 'post',
			'has_archive'        => true,
			'hierarchical'       => false,
			'menu_position'      => null,
			'supports'           => array(  'author' ),
			'menu_icon'			 => 'dashicons-editor-unlink',

		);

		register_post_type( 'lar_link', $args );
	}

	/**
	 * The plugin use this method to register the custom taxonomies of the links.
	 * @since    1.0.0
	 */
	public function register_taxonomies(){
		$labels = array(
			'name'                       => _x( 'Link Categories', 'taxonomy general name' ),
			'singular_name'              => _x( 'Link Category', 'taxonomy singular name' ),
			'search_items'               => __( 'Search Link Categories', 'affiliate-butler-pro' ),
			'popular_items'              => __( 'Popular Link Categories', 'affiliate-butler-pro' ),
			'all_items'                  => __( 'All Link Categories','affiliate-butler-pro' ),
			'parent_item'                => null,
			'parent_item_colon'          => null,
			'edit_item'                  => __( 'Edit Link Category','affiliate-butler-pro' ),
			'update_item'                => __( 'Update Link Category','affiliate-butler-pro' ),
			'add_new_item'               => __( 'Add New Link Category','affiliate-butler-pro' ),
			'new_item_name'              => __( 'New Link Category Name','affiliate-butler-pro' ),
			'separate_items_with_commas' => __( 'Separate link categories with commas','affiliate-butler-pro' ),
			'add_or_remove_items'        => __( 'Add or remove link category','affiliate-butler-pro' ),
			'choose_from_most_used'      => __( 'Choose from the most used links categories','affiliate-butler-pro' ),
			'not_found'                  => __( 'No links categories found.','affiliate-butler-pro' ),
			'menu_name'                  => __( 'Link Categories','affiliate-butler-pro' ),
		);

		$args = array(
			'hierarchical'          => true,
			'labels'                => $labels,
			'show_ui'               => true,
			'show_admin_column'     => true,
			'update_count_callback' => '_update_post_term_count',
			'query_var'             => true,
			'rewrite'               => array( 'slug' => 'models' ),
		);

		register_taxonomy( 'links_category', 'lar_link', $args );
	}


	/**
	 *	Using CMB2 library The method will build a custom meta box for the user to fill the link information.
	 *
	 *  @since    1.0.0
	 */
	public function lar_links_register_metabox() {


		$add_links_box = new_cmb2_box( array(
			'id'            => ABUTLER_PRO_PLUGIN_PREFIX . 'metabox',
			'title'         => __( 'Add new Link', 'affiliate-butler-pro' ),
			'object_types'  => array( 'lar_link', ), // Post type
			'context'       => 'normal',
			'priority'      => 'high',
			'show_names'    => true, // Show field names on the left
		) );


		$add_links_box->add_field( array(
			'name' => __( 'Keyword/s', 'affiliate-butler-pro' ),

			'id'   => ABUTLER_PRO_PLUGIN_PREFIX . 'keywords',
			'type' => 'text_medium',
			'repeatable' => true,
		) );

		$add_links_box->add_field( array(
			'name' => __( 'Link Type', 'affiliate-butler-pro' ),
			'id'   => ABUTLER_PRO_PLUGIN_PREFIX . 'link_type',
			'type' => 'select',
			'options' => array(
					  'external' => __('External','affiliate-butler-pro'),
					  'internal' => __('Internal','affiliate-butler-pro'),
					  'popup' => __('Popup','affiliate-butler-pro'),
					  'popup_image' => __('Popup Image','affiliate-butler-pro'),
					  'popup_gallery' => __('Popup Gallery','affiliate-butler-pro'),
					  'popup_video' => __('Popup Video','affiliate-butler-pro'),
					  'popup_map' => __('Google Map','affiliate-butler-pro'),
					  'url_share' => __('Shared URL','affiliate-butler-pro'),
				),
			'description' => '',
		) );


		$add_links_box->add_field( array(
			'name' => __( 'URL (Link)', 'affiliate-butler-pro' ),
			'default' => 'http://',
			'id'   => ABUTLER_PRO_PLUGIN_PREFIX . 'url',
			'type' => 'text_url',
		) );


		$add_links_box->add_field( array(
			'name' => __( 'Link Title', 'affiliate-butler-pro' ),
			'id'   => ABUTLER_PRO_PLUGIN_PREFIX . 'link_title',
			'type' => 'text_medium',
			'description' => __('For SEO','affiliate-butler-pro'),
		) );



		$add_links_box->add_field( array(
			'name' => __( 'Internal URL (Link)', 'affiliate-butler-pro' ),
			'id'   => ABUTLER_PRO_PLUGIN_PREFIX . 'internal_url',
			'type' => 'select',
			'options' => array('0' => __('Select Post/Page','affiliate-butler-pro' ))
		) );

		$add_links_box->add_field( array(
		    'name'    => 'Popup Content',
		    'desc'    => 'Fill in the content you want to appear in the poup',
		    'id'      => ABUTLER_PRO_PLUGIN_PREFIX . 'popup_content',
		    'type'    => 'wysiwyg',
		    'sanitization_cb' => false,
		    'options' => array(
		    	'wpautop' => false,
		    	'teeny' => true,
		    	'textarea_rows' => 5,
		    	),
		) );

		$add_links_box->add_field( array(
		    'name'    => 'Popup Image',
		    'desc'    => 'Upload an image or enter an URL.',
		    'id'      => ABUTLER_PRO_PLUGIN_PREFIX . 'popup_image',
		    'type'    => 'file',
		    // Optional:
		    'options' => array(
		        'url' => false, // Hide the text input for the url
		        'add_upload_file_text' => 'Upload Image' // Change upload button text. Default: "Add or Upload File"
		    ),
		) );

		$add_links_box->add_field( array(
		    'name' => 'Gallery images',
		    'desc' => '',
		    'id'   => ABUTLER_PRO_PLUGIN_PREFIX . 'popup_gallery',
		    'type' => 'file_list',
		    'preview_size' => array( 100, 100 ), // Default: array( 50, 50 )
		    // Optional, override default text strings
		    'options' => array(
		        'add_upload_files_text' => 'Add or Upload Images', // default: "Add or Upload Files"
		    ),
		) );


		$add_links_box->add_field( array(
		    'name' => __( 'Video Url', 'affiliate-butler-pro' ),
		    'id'   => ABUTLER_PRO_PLUGIN_PREFIX . 'popup_videourl',
		    'type' => 'text_url',
		    'desc' => __('Only supports Youtube and Vimeo. Examples: http://www.youtube.com/watch?v=7HKoqNJtMTQ or https://vimeo.com/129187534','affiliate-butler-pro'),
		    'protocols' => array( 'http', 'https' ), // Array of allowed protocols
		) );

		$add_links_box->add_field( array(
		    'name' => __( 'Map Url', 'affiliate-butler-pro' ),
		    'id'   => ABUTLER_PRO_PLUGIN_PREFIX . 'popup_mapurl',
		    'type' => 'text_url',
		    'desc' => __('Only supports Google Map. URL Example: https://maps.google.com/maps?q=221B+Baker+Street,+London,+United+Kingdom&hl=en','affiliate-butler-pro'),
		    'protocols' => array( 'http', 'https' ), // Array of allowed protocols
		) );

		// Sharing fields
		$add_links_box->add_field( array(
		    'name' => __( 'Shared URL', 'affiliate-butler-pro' ),
		    'id'   => ABUTLER_PRO_PLUGIN_PREFIX . 'shared_url',
		    'type' => 'text_url',
		    'desc' => __('The URL which you want to be shared when the user click on the keyword.','affiliate-butler-pro'),
		    'protocols' => array( 'http', 'https' ), // Array of allowed protocols
		) );
		$add_links_box->add_field( array(
			'name' => __( 'Shared Text', 'affiliate-butler-pro' ),
			'id'   => ABUTLER_PRO_PLUGIN_PREFIX . 'shared_text',
			'type' => 'textarea_small',
			'description' => __('The Text which you want to be shared along with the URL.','affiliate-butler-pro'),

		));



		////////////////


		$add_links_box->add_field( array(
			'name' => __( 'Dofollow?', 'affiliate-butler-pro' ),
			'id'   => ABUTLER_PRO_PLUGIN_PREFIX . 'do_follow',
			'type' => 'checkbox',
			#'default'=> abutler_pro()->get_option(ABUTLER_PRO_PLUGIN_PREFIX . 'do_follow'),
			'description' => __('if you checked this option, you will allow search engines to follow this link and use it in ranking.','affiliate-butler-pro'),
		) );

		$add_links_box->add_field( array(
			'name' => __( 'Open in:', 'affiliate-butler-pro' ),
			'id'   => ABUTLER_PRO_PLUGIN_PREFIX . 'open_in',
			'type' => 'select',
			'default'=> abutler_pro()->get_option(ABUTLER_PRO_PLUGIN_PREFIX . 'open_in'),
			'options' => array(
					  '_self' => __('Same Window','affiliate-butler-pro'),
					  '_blank' => __('New Window','affiliate-butler-pro'),
				),
			'description' => __('When the visitor click on the link, it\'s either open in a new window or the same window.','affiliate-butler-pro'),
		) );

		$links_prefix = (abutler_pro()->get_option(ABUTLER_PRO_PLUGIN_PREFIX . 'shortlinks_prefix'))?abutler_pro()->get_option(ABUTLER_PRO_PLUGIN_PREFIX . 'shortlinks_prefix'):'go';

		$add_links_box->add_field( array(
			'name' => __( 'Shrink?', 'affiliate-butler-pro' ),
			'id'   => ABUTLER_PRO_PLUGIN_PREFIX . 'shrink',
			'type' => 'checkbox',
			#'default'=> abutler_pro()->get_option(ABUTLER_PRO_PLUGIN_PREFIX . 'shrink'),
			'description' => sprintf( __('The link will be shortened (e.g example.com/%s/amazon)','affiliate-butler-pro'),$links_prefix),
		) );




		$add_links_box->add_field( array(
			'name' => __( 'Slug', 'affiliate-butler-pro' ),
			'default' => $this->last_link_id,
			'id'   => ABUTLER_PRO_PLUGIN_PREFIX . 'slug',
			'type' => 'text_small',
			'description' => __('The slug for the shortened link','affiliate-butler-pro').' <span id="abutler_slug_example"></span>',
		));



		$add_links_box->add_field( array(
			'name' => __( 'Case Sensitive?', 'affiliate-butler-pro' ),

			'id'   => ABUTLER_PRO_PLUGIN_PREFIX . 'is_sensitive',
			'type' => 'checkbox',
			//'default'=> abutler_pro()->get_option(ABUTLER_PRO_PLUGIN_PREFIX . 'is_sensitive'),
			'description' => __('If you checked this option, the plugin will replace the keywords exactly according to the letters case.','affiliate-butler-pro').' <span id="abutler_slug_example"></span>',
		));



	}

	/**
	 * Ajax called action to validate the link data of the meta box.
	 * @since    1.0.0
	 */
	public function pre_submit_link_validation(){
		$errors = array();
	    //simple Security check
	    check_ajax_referer( 'my_pre_submit_validation', 'security' );


	    parse_str($_POST['form_data'], $link);
	    $keywords = array_filter($link[ABUTLER_PRO_PLUGIN_PREFIX.'keywords']);
	    $keywords = array_map('trim', $keywords);
	    if( count(array_unique($keywords)) < count($keywords) ){
	    	$errors['keywords'] = __('Please, remove the repetition from the keywords','affiliate-butler-pro');
	    }



	    if(empty($keywords))
	    {
	    	$errors['keywords'] = __('Please provide keyword/s','affiliate-butler-pro');
	    }
	    $link_type = isset($link[ABUTLER_PRO_PLUGIN_PREFIX . 'link_type'])?$link[ABUTLER_PRO_PLUGIN_PREFIX . 'link_type']:'';
	    $link_url = isset($link[ABUTLER_PRO_PLUGIN_PREFIX . 'url'])?$link[ABUTLER_PRO_PLUGIN_PREFIX . 'url']:'';
	    if($link_type == 'external' OR $link_type ==''){
			if($link_url == '' OR parse_url($link_url, PHP_URL_HOST) === NULL)
			{
			    $errors['url'] = __('Please provide a valid url','affiliate-butler-pro');
			}
		}

		if($link[ABUTLER_PRO_PLUGIN_PREFIX . 'link_type'] === 'internal' and !is_numeric($link[ABUTLER_PRO_PLUGIN_PREFIX . 'internal_url'])){
			    $errors['internal_url'] = __('Choose internal link please.','affiliate-butler-pro');

	    }

	    // we don't want to touch the DB unless the user fill all the data right.
	    if(empty($errors)){

		    $keywords = $this->get_meta_values(ABUTLER_PRO_PLUGIN_PREFIX . 'keywords', 'lar_link','publish',$link['post_ID']);
		    foreach ($keywords as $key => $value) {
		   		$intersect = array_intersect($link[ABUTLER_PRO_PLUGIN_PREFIX.'keywords'], unserialize($value));
		   		if(!empty($intersect)){
		   			$errors['keywords'] = 'keyword\s ( '. implode(',', $intersect) .' ) is already exist';
		   			break;
		   		}
		    }


		    $urls = $this->get_meta_values(ABUTLER_PRO_PLUGIN_PREFIX . 'url', 'lar_link','publish',$link['post_ID']);
		    if($link[ABUTLER_PRO_PLUGIN_PREFIX . 'link_type'] == 'external'){
			    if(in_array($link[ABUTLER_PRO_PLUGIN_PREFIX . 'url'], $urls)){
			    	$errors['url'] = __('URL is already exist','affiliate-butler-pro');
			    }
			}

			$link_slug = (isset($link[ABUTLER_PRO_PLUGIN_PREFIX . 'slug']))?$link[ABUTLER_PRO_PLUGIN_PREFIX . 'slug']:'';
			if(trim($link_slug)!=''){
			    $slugs = $this->get_meta_values(ABUTLER_PRO_PLUGIN_PREFIX . 'slug', 'lar_link','publish',$link['post_ID']);
			    if(in_array($link_slug, $slugs)){
			    	$errors['slugs'] = sprintf(__( 'Slug (%s) is already exist','affiliate-butler-pro'),$link_slug);
			    }
			}
		} //empty($errors)

	    if(!empty($errors)){
	    	echo 'Please correct the following errors: <ul id="abutler_errors"><li>';
	    	echo implode('</li><li>', $errors);
	    	echo '</li></ul>';
	    }else{
	    	add_option('last_lar_link_id',$link['post_ID']) or update_option('last_lar_link_id',$link['post_ID']);
	    	echo '1';
	    }

	    die();
	}



	/**
	 * A helper method to get all the values sored in wp_postmeta table that has one key.
	 * @param	 string the meta key.
	 * @param	 string the post type
	 * @param	 string the post status
	 * @param	 number the post ID to exlude from the query
	 * @return 	 stdObject	All the rows with the specified key.
	 * @since    1.0.0
	 */
	public function get_meta_values( $key = '', $type = 'post', $status = 'publish', $post_ID = 0 ) {

	    global $wpdb;

	    if( empty( $key ) )
	        return;

	    $r = $wpdb->get_col( $wpdb->prepare( "
	        SELECT pm.meta_value FROM {$wpdb->postmeta} pm
	        LEFT JOIN {$wpdb->posts} p ON p.ID = pm.post_id
	        WHERE pm.meta_key = '%s'
	        AND p.post_status = '%s'
	        AND p.post_type = '%s'
	        AND p.ID <> ".$post_ID
	        , $key, $status, $type ) );

	    return $r;
	}
	/**
	 * This one is used to enqueue the scripts into our link post type page.
	 * @since    1.0.0
	 */
	public function insert_validation_nonce(){
		$links_prefix = (abutler_pro()->get_option(ABUTLER_PRO_PLUGIN_PREFIX . 'shortlinks_prefix'))?abutler_pro()->get_option(ABUTLER_PRO_PLUGIN_PREFIX . 'shortlinks_prefix'):'go';
		if(isset($_GET['post'])){
	 		$link_slug = get_post_meta($_GET['post'], ABUTLER_PRO_PLUGIN_PREFIX.'slug',true);
		}

	 	?>
		 	<script type="text/javascript">
		 		<?php do_action('abutler_add_js_variables'); ?>
		 		var validation_nonce = '<?php echo wp_create_nonce( 'my_pre_submit_validation' ); ?>';
		 		var plugin_prefix = '<?php echo ABUTLER_PRO_PLUGIN_PREFIX; ?>';
		 		<?php if(isset($link_slug)): ?>
		 			var last_link_id = '<?php echo $link_slug; ?>';
		 		<?php else: ?>
		 			var last_link_id = '<?php echo $this->last_link_id; ?>';
		 		<?php endif; ?>
		 		var home_url = '<?php echo home_url(); ?>';
		 	</script>
	 	<?php
	 	wp_enqueue_script( $this->Affiliate_Butler_Pro.'-validation', plugin_dir_url( __FILE__ ) . 'js/abutler-links-validation.js', array( 'jquery' ), $this->version, false );
	 	wp_localize_script( $this->Affiliate_Butler_Pro.'-validation', 'abutler_vars', array(
	 		'links_prefix' => $links_prefix,
	 	));


	}



	/**
	 * Adding a new meta box to disable the auto-replacement for a specific post,page.
	 *
	 * @since    1.0.0
	 */
	public function disable_for_single_post(){
		global $abutler_name;
		$screens = apply_filters('abutler_disable_box_screens',array( 'post', 'page' ));

	    foreach ( $screens as $screen ) {
	        add_meta_box( 'abutler_meta', __( 'Disable '. $abutler_name .' for this post', 'affiliate-butler-pro' ), array($this,'abutler_meta_callback'), $screen );
	    }
	}
	/**
	 * Adding a new meta box to disable the auto-replacement for a specific post,page.
	 *
	 * @since    1.1.0
	 */
	public function keyword_stats_box(){
		global $abutler_name;
		add_meta_box( 'abutler_keyword_stats_meta', __( 'Keyword/s Stats', 'affiliate-butler-pro' ), array($this,'abutler_keyword_stats_meta_callback'), 'lar_link' );
	}
	/**
	 * The callback of the disable meta box.
	 * @param	 StdObject the post object
	 * @since    1.0.0
	 */
	public function abutler_keyword_stats_meta_callback( $post ) {
		global $wpdb;
	    $keywords = get_post_meta( $post->ID, ABUTLER_PRO_PLUGIN_PREFIX . 'keywords', true );
	    $is_sensitive = get_post_meta( $post->ID, ABUTLER_PRO_PLUGIN_PREFIX . 'is_sensitive', true );
	    if($keywords != ''){
		    foreach($keywords as $keyword):
		    	if($is_sensitive === 'on')
		    		$posts = $wpdb->get_results(" SELECT ID,post_title,post_content FROM ".$wpdb->posts." as p WHERE (p.post_content LIKE BINARY '% ".$keyword." %' OR p.post_content LIKE BINARY '%".$keyword." ' OR p.post_content LIKE BINARY '".$keyword." %' OR p.post_content LIKE BINARY '".$keyword."') AND p.post_type <> 'revision' AND p.post_type <> 'attachment'  AND p.post_type <> 'nav_menu_item' AND p.post_type <> 'lar_link' ");
		    	else
		    		$posts = $wpdb->get_results(" SELECT ID,post_title,post_content FROM ".$wpdb->posts." as p WHERE (p.post_content LIKE  '% ".$keyword." %' OR p.post_content LIKE  '%".$keyword." ' OR p.post_content LIKE  '".$keyword." %' OR p.post_content LIKE '".$keyword."') AND p.post_type <> 'revision' AND p.post_type <> 'attachment'  AND p.post_type <> 'nav_menu_item' AND p.post_type <> 'lar_link' ");

			    ?>
			    <?php if($posts): ?>
				 	<h4><?php _e('Keyword', 'affiliate-butler-pro'); ?>: <?php echo $keyword; ?></h4>
					<table class="list-posts widefat">
						<thead>
							<th><?php _e('Content', 'affiliate-butler-pro'); ?></th>
							<th><?php _e('Occurrences', 'affiliate-butler-pro'); ?></th>
							<th><?php _e('Post Type', 'affiliate-butler-pro'); ?></th>
						</thead>
						<?php foreach($posts as $post): ?>
						<?php $post_type = get_post_type_object( get_post_type($post->ID) ); ?>
							<tr>
								<td><a href="<?php echo get_edit_post_link( $post->ID ); ?>"><?php echo $post->post_title; ?></a></td>
								<td><?php echo $this->count_occurencies(strtoupper($post->post_content), strtoupper($keyword)); ?></td>
								<td><?php echo (isset($post_type->labels->singular_name))?$post_type->labels->singular_name:ucwords(str_replace('_', ' ', get_post_type($post->ID))); ?></td>
							</tr>
						<?php endforeach; ?>
					</table>
		 		<?php endif; ?>
		    <?php endforeach;
	    }else{
	    	echo '<p>' . __('Stats will appear after saving the Auto-Link','affiliate-butler-pro') . '</p>';
	    }
	}
	/**
	 * The callback of the disable meta box.
	 * @param	 StdObject the post object
	 * @since    1.0.0
	 */
	public function abutler_meta_callback( $post ) {
	    wp_nonce_field( basename( __FILE__ ), 'abutler_nonce' );

	    $abutler_disabled_meta = get_post_meta( $post->ID );

	    ?>

	    <p>
	        <input type="checkbox" name="abutler_disabled" id="abutler_disabled" <?php if (isset($abutler_disabled_meta['abutler_disabled']) &&   $abutler_disabled_meta['abutler_disabled'][0] == 'on' ) echo 'checked="checked"'; ?> />
	   		<label for="abutler_disabled" class="abutler-row-title"><?php _e( 'Disable', 'abutler-affiliate-butler-pro' )?></label>
	    </p>

	    <?php
	}

	/**
	 * Saving the `disable` option for each post,page.
	 * @param	 integer the post ID.
	 * @since    1.5.0
	 */
	function abutler_meta_save( $post_id ) {
 		// If post_type is not post or page, just do nothing.
		$screens = apply_filters('abutler_disable_box_screens',array( 'post', 'page' ));

 		if(!in_array(get_post_type($post_id), $screens)) return;
	    // Checks save status
	    $is_autosave = wp_is_post_autosave( $post_id );
	    $is_revision = wp_is_post_revision( $post_id );
	    $is_valid_nonce = ( isset( $_POST[ 'abutler_nonce' ] ) && wp_verify_nonce( $_POST[ 'abutler_nonce' ], basename( __FILE__ ) ) ) ? 'true' : 'false';

	    // Exits script depending on save status
	    if ( $is_autosave || $is_revision || !$is_valid_nonce ) {
	        return;
	    }

	    // Checks for input and sanitizes/saves if needed
	    if( isset( $_POST[ 'abutler_disabled' ] ) ) {

	        update_post_meta( $post_id, 'abutler_disabled',  'on'  );
	    }else{
	        update_post_meta( $post_id, 'abutler_disabled',  'off'  );

	    }
	}


	/**
	* Change the default Wordpress colums heads for links post types.
	* @param	 array default admin colums heads.
	* @return	 array altered colums heads.
	* @since    1.0.0
	*/
	public function abutler_columns_head( $defaults ) {
			unset($defaults['title']);
			unset($defaults['author']);
		    $new = array();
			foreach($defaults as $key => $title) {
			    if ($key==apply_filters('abutler_put_colums_before','date')){ // Put the Thumbnail column before the Author column
			      		$new['keywords'] = __('Keyword/s','affiliate-butler-pro');
		    			$new['link'] = __('Preview','affiliate-butler-pro');
		    			$new['total_clicks'] = __('Total Clicks','affiliate-butler-pro');
		    			$new = apply_filters('lar_links_colums_heads',$new);
			  		}
			    	$new[$key] = $title;
			}

			return $new;


	}

	/**
	* Change the default Wordpress colums for links post types.
	* @param	 array default admin colums.
	* @return	 array altered colums.
	* @since    1.0.0
	**/
	public function abutler_columns_content($column_name, $post_ID) {

		    if($column_name == 'link'){
		    	echo Abutler_Link_Pro::get_final_url($post_ID);
		    }

		    if ($column_name == 'total_clicks') {
		        $stats = get_post_meta($post_ID, ABUTLER_PRO_PLUGIN_PREFIX.'stats');
		        $clicks_stat = (isset($stats[0]['total_visits']))?$stats[0]['total_visits']:'0';
		        echo $clicks_stat;

		    }

		    if($column_name == 'keywords'){
		    	$keywords = get_post_meta($post_ID, ABUTLER_PRO_PLUGIN_PREFIX.'keywords',true);
		    	$stats = get_post_meta($post_ID, ABUTLER_PRO_PLUGIN_PREFIX.'stats');

		    	if(!empty($keywords)){
		    		?>
		    		<strong><a class="row-title" href="<?php echo get_edit_post_link($post_ID);  ?>" title="Edit"><?php echo implode(' | ',$keywords); ?></a></strong>
			    		<div class="row-actions">
				    		<span class="edit"><a href="<?php echo get_edit_post_link($post_ID);  ?>" title="Edit this item">Edit</a> | </span>
				    		<span class="trash"><a class="submitdelete" title="Move this item to the Trash" href="<?php echo get_delete_post_link($post_ID); ?>">Trash</a>  </span>
				    		<span class="quick_stats" class="edit"> | <a href="<?php echo admin_url('admin.php?page=abutler_upgrade_settings'); ?>"><?php _e('Stats (PRO)','affiliate-butler-pro'); ?></a> </span>
				    		<?php do_action('abutler_add_quick_links', $post_ID); ?>

			    		</div>

		    		<?php

		    	}else{
		    		echo '-';
		    	}

		    }
	}

	/**
	 * Adding a new meta box for promoting the pro version
	 *
	 * @since    2.0.1
	 */
	public function add_upgrade_to_pro_box(){
		global $abutler_name;
	    add_meta_box( 'abutler_pro_metabox', __( 'Upgrade to PRO', 'affiliate-butler-pro' ), array($this,'meta_pro_callback'), 'lar_link', 'side' );
	}

	public function meta_pro_callback($post){
		echo '<a href="https://goo.gl/ILo1iY" target="_blank"><img style="width:100%;" src="'.ABUTLER_URL.'admin/images/upgrade_to_pro.png" /></a>';
	}


	/**
	 * This one is used to enqueue the scripts into our link post type page.
	 * @since    1.0.0
	 */
	//@filter
	public function add_internal_variables(){
		if(isset($_GET['post'])){
		 	$link_internal = get_post_meta($_GET['post'], ABUTLER_PRO_PLUGIN_PREFIX.'internal_url',true);

		 	?>

			var internal_id = <?php echo ($link_internal)?$link_internal:"''"; ?>;
			var internal_title = '<?php echo get_the_title($link_internal); ?>';
			<?php
		}
	}



	/**
	* Add the scripts of the stats page (the map and the chartjs library)
	*
	* @since    1.0.0
	*/
	public function insert_jqvmap(){

	 	wp_enqueue_script( $this->Affiliate_Butler_Pro.'-vmap', plugin_dir_url( __FILE__ ) . 'js/jqvmap/jquery.vmap.min.js', array( 'jquery' ), $this->version, false );
	 	wp_enqueue_script( $this->Affiliate_Butler_Pro.'-vmapworld', plugin_dir_url( __FILE__ ) . 'js/jqvmap/maps/jquery.vmap.world.js', array( 'jquery' ), $this->version, false );

	 	wp_enqueue_script( $this->Affiliate_Butler_Pro.'-vmap-init', plugin_dir_url( __FILE__ ) . 'js/jqvmap/jquery.vmap.init.js', array( 'jquery' ), $this->version, false );
	 	// add chartjs
	 	wp_enqueue_script( $this->Affiliate_Butler_Pro.'-chartjs', plugin_dir_url( __FILE__ ) . 'js/chartjs/Chart.min.js', array( 'jquery' ), $this->version, false );
	 	wp_enqueue_script( $this->Affiliate_Butler_Pro.'-chartjs-init', plugin_dir_url( __FILE__ ) . 'js/chartjs.init.js', array( $this->Affiliate_Butler_Pro.'-chartjs' ), $this->version, false );


	}




	/**
	* Filter to change the column that our custom colums arranged before it
	* @param	 string old column
	* @return	 strnig new column
	* @since    1.0.0
	*/
	public function before_column($column){
		return 'taxonomy-links_category';
	}



	/**
	* Add the stats page link to the row actions for Auto Links post type.
	* @param	 int post ID
	* @return	 void
	* @since    1.0.0
	*/
	public function add_stats_link($post_ID){
		$stats = get_post_meta($post_ID, ABUTLER_PRO_PLUGIN_PREFIX.'stats');

		if(isset($stats[0]['total_visits']) && $stats[0]['total_visits']>0): ?><span class="stats"> | <a title="View Stats" href="<?php echo Abutler_Stats::get_stats_link($post_ID); ?>"><?php _e('Stats Details','affiliate-butler-pro'); ?></a> </span><?php endif;

	}


	/**
	*
	* This method is called through Ajax request, it's used to search for posts,pages for internal links.
	* @since    1.0.0
	*
	**/
	public function search_internal_linking(){
	    global $wpdb;
		$s = urldecode($_GET['q']);
		$post_types = array_keys(get_post_types(array('public' => true)));
		unset($post_types['lar_link']);
		$posts = $wpdb->get_results(
		    sprintf("SELECT id,post_title FROM %s WHERE post_title LIKE '%%%s%%' AND post_type IN (%s)", $wpdb->posts, $s, "'" .implode("','", $post_types) . "'")
		);
		$result  = array();
		$i = 0;
		foreach($posts as $p){
			$single = array();
			$single['id'] = $p->id;
			$single['text'] = sprintf('%s (%s)', $p->post_title, ucfirst(get_post_type($p->id)));
			array_push($result, $single);
			$i++;
		}

		echo json_encode($result);
		exit;


	}


	/**
	* Add the stats page
	* @since     1.0.0
	**/
	public function add_stats_page(){
		add_submenu_page( null, __('Link Stats','affiliate-butler-pro'), __('Link Stats','affiliate-butler-pro'), 'manage_options', 'lar_link_stats' , array($this, 'display_link_stats'));
	}

	/**
	* Include the stats template
	* @since     1.0.0
	**/
	public function display_link_stats(){
		include plugin_dir_path(__FILE__).'partials/link_stats.php';
	}

	/**
	* Show `error` or `success` messages in the admin
	* @param	 string the user's message.
	* @param	 string message type (`error`,`updated`)
	* @since    1.0.0
	**/
	private function show_message( $message, $type){
		echo '<div class="' . $type . '"><p>'. $message .'</p></div>';
	}

	/**
	* Remove non-required action links in the Links table.
	* @param	 array the action array
	* @param	 object post Object
	* @since    1.0.0
	**/
	public function remove_row_actions( $actions, $post )
	{
	  global $current_screen;
		if( $current_screen->post_type != 'lar_link' ) return $actions;
		unset( $actions['edit'] );
		unset( $actions['view'] );
		unset( $actions['trash'] );
		unset( $actions['inline hide-if-no-js'] );

		return $actions;
	}

	/**
	* Redirect the user to the plugin's docs.
	* @return void()
	* @since    1.1.0
	**/
	public function add_abutler_docs_link_to_menu(){
		global $submenu;
		if( isset($submenu['edit.php?post_type=lar_link'])){
			$submenu['edit.php?post_type=lar_link'][20] = array( __('Documentation', 'affiliate-butler-pro'), 'manage_options' , 'https://wpruby.com/knowledgebase_category/affiliate-butler-pro/' );
		}
	}

	/**
	 * Calculate the occurencies of a word in a string.
	 * @param	 string The text
	 * @param	 string The word
	 * @return	 integer Number of occurencies
	 * @since    1.1.0
	 */
	private function count_occurencies($str, $word) {
		$occurencies = array_count_values(str_word_count($str, 1));
		if(isset($occurencies[$word])){
			return $occurencies[$word];
		}else{
			return 0;
		}
 	}

}
