<?php 


class Abutler_Pro_Settings{


	protected $tabs = array();
	protected $key = 'abutler_settings';



	/**
	* Define the pages in the constructor.
	* @since     1.0.0
	**/
	public function __construct(){

		$this->tabs = apply_filters('abutler_settings_tabs',array(

				'abutler_settings' => array(
						'id' => 'abutler_settings',
						'title' => __('Settings','affiliate-butler-pro'),
						'metabox_callback' => false,
						'callback' => array($this , 'admin_page_display')
					),
				));

		$this->hooks();

	}



	/**
	* Init the settings hooks.
	* @since     1.0.0
	**/
	public function hooks(){
		add_action( 'admin_init', array( $this, 'init' ) );
		add_action( 'admin_menu', array( $this, 'add_options_page' ) );
		add_action('cmb2_init',array($this,'add_main_settings_page_metabox'));
		add_action('cmb2_save_options-page_fields', array($this, 'refresh_rules'), 10, 3);
		foreach($this->tabs as $tab){
			if($tab['metabox_callback'] !== false){
				add_action( 'cmb2_init',  $tab['metabox_callback']);
			}
		}
	}

	/**
	* register the plugin's settings.
	* @since     1.0.0
	**/
	public function init() {
		register_setting( $this->key, $this->key );
	}


	/**
	* Register the plugin settings page.
	* @since     1.0.0
	**/
	public function add_options_page() {
		global $abutler_name;

		//$this->settings_page =   add_menu_page($abutler_name,$abutler_name,'manage_options',$this->key,array($this, 'admin_page_display'),'',55.05);

	    foreach ($this->tabs as $key => $tab){
			if(isset($tab['callback']))
				add_submenu_page( 'edit.php?post_type=lar_link', $tab['title'], $tab['title'], 'manage_options', $key , $tab['callback']);
			else
				add_submenu_page( 'edit.php?post_type=lar_link', $tab['title'], $tab['title'], 'manage_options', $key , array($this, 'admin_page_display'));

		}
			 
	
		do_action('abutler_add_custom_pages');
		
	}

	/**
	* Include main settings page template
	* @since     1.0.0
	**/
	public function admin_page_display(){
		// include partial
		require_once ABUTLERPRO_DIR . 'admin/partials/settings.php';
		
	}


	/**
	* Include upgrade page template
	* @since     1.0.0
	**/
	public function display_upgrade_page(){
		require_once ABUTLERPRO_DIR . 'admin/partials/upgrade.php';

	}



	/**
	* The meta box fields of the upgrade settings page
	* @since     1.0.0
	**/
	public function add_upgrade_settings_page_metabox(){

		$cmb = new_cmb2_box( array(
			'id'      => 'abutler_upgrade_settings',
			'hookup'  => false,
			'show_on' => array(
				// These are important, don't remove
				'key'   => 'options-page',
				'value' => array( $this->key, )
			),
		) );

		


	}



	/**
	* The meta box fields of the main settings page
	* @since     1.0.0
	**/
	public function add_main_settings_page_metabox(){

		$links_prefix = (abutler_pro()->get_option(ABUTLER_PRO_PLUGIN_PREFIX . 'shortlinks_prefix'))?abutler_pro()->get_option(ABUTLER_PRO_PLUGIN_PREFIX . 'shortlinks_prefix'):'go';

		$cmb = new_cmb2_box( array(
			'id'      => 'abutler_settings',
			'hookup'  => false,
			'show_on' => array(
				// These are important, don't remove
				'key'   => 'options-page',
				'value' => array( $this->key, )
			),
		) );


		// Set our CMB2 fields
		$cmb->add_field( array(
			'name'    => __( 'Enable Auto Replcement', 'affiliate-butler-pro' ),
			'id'      => ABUTLER_PRO_PLUGIN_PREFIX . 'enable',
			'type'    => 'checkbox',
			'default' => $this->get_if_enabled( true )
		) );
		$cmb->add_field( array(
			'name'    => __( '<h2>"Add Auto Link" Defaults</h2>', 'affiliate-butler-pro' ),
			'id'	  => 'default_settings',
			'type'    => 'title',
			//'default' => 'yes',
		) );
	

		$cmb->add_field( array(
			'name' => __( 'Dofollow?', 'affiliate-butler-pro' ),
			'id'   => ABUTLER_PRO_PLUGIN_PREFIX . 'do_follow',
			'type' => 'checkbox',
			'description' => __('if you checked this option, you will allow search engines to follow this link and use it in ranking.','affiliate-butler-pro'),
		) );

		$cmb->add_field( array(
			'name' => __( 'Open in:', 'affiliate-butler-pro' ),
			'id'   => ABUTLER_PRO_PLUGIN_PREFIX . 'open_in',
			'type' => 'select',
			'default' => '_self',
			'options' => array(
					  '_self' => __('Same Window','affiliate-butler-pro'),
					  '_blank' => __('New Window','affiliate-butler-pro'),
				),
		) );


		$cmb->add_field( array(
			'name' => __( 'Shrink?', 'affiliate-butler-pro' ),
			'id'   => ABUTLER_PRO_PLUGIN_PREFIX . 'shrink',
			'type' => 'checkbox',
			'description' => sprintf( __('The link will be shortened (e.g example.com/%s/amazon)','affiliate-butler-pro'),$links_prefix),
		) );



		$cmb->add_field( array(
			'name' => __( 'Case Sensitive?', 'affiliate-butler-pro' ),
			'id'   => ABUTLER_PRO_PLUGIN_PREFIX . 'is_sensitive',
			'type' => 'checkbox',
			'description' => __('If you checked this option, the plugin will replace the keywords exactly according to the letters case.','affiliate-butler-pro'),
		));
		
		$cmb->add_field( array(
			'name'    => __( '<h2>General Settings</h2>', 'affiliate-butler-pro' ),
			'id'	  => 'general_settings',
			'type'    => 'title',
			//'default' => 'yes',
		) );


		$cmb->add_field( array(
		    'name'    => __( 'Auto Link taxonomies?', 'affiliate-butler-pro' ),
		    'desc'    => __('Auto link categories and taxonomies terms to their archive pages whenever they appear in posts or pages.','affiliate-butler-pro'),
		    'id'      => ABUTLER_PRO_PLUGIN_PREFIX . 'link_categories',
		    'type'    => 'multicheck',
		    'options' => $this->get_taxonomies()
		 	));
		// since 2.1.1
		$cmb->add_field( array(
		    'name'    => __( 'Limits Per Post', 'affiliate-butler-pro' ),
		    'desc'    => __('Times (Keep it empty to set it to make it unlimited)','affiliate-butler-pro'),
		    'id'      => ABUTLER_PRO_PLUGIN_PREFIX . 'per_post_limit',
		    'type'    => 'text_small',
		    'default' => '',
		    'sanitization_cb' => array($this, 'validate_per_post_limit'),
		 	));
		$cmb->add_field( array(
		    'name'    => __( 'Excluded HTML tags', 'affiliate-butler-pro' ),
		    'desc'    => __('These HTML tags will be ignored if keywords are located into them. Seperate the tags by comma. e.g. h1,h2,strong ','affiliate-butler-pro'),
		    'id'      => ABUTLER_PRO_PLUGIN_PREFIX . 'excluded_html_tags',
		    'type'    => 'text',
		    'default' => '',
		 	));
		$cmb->add_field( array(
		    'name'    => __( 'Short Links Prefix', 'affiliate-butler-pro' ),
		    'desc'    => sprintf(__('Currently, the short links would be in this format %s/%s/{url_slug}','affiliate-butler-pro'), home_url(), '<strong>'.$links_prefix.'</strong>' ) ,
		    'id'      => ABUTLER_PRO_PLUGIN_PREFIX . 'shortlinks_prefix',
		    'type'    => 'text_small',
		    'default' => 'go',
		 	));	
		$cmb->add_field( array(
			'name' => __( 'Disable Pages/Posts self liniking', 'affiliate-butler-pro' ),
			'id'   => ABUTLER_PRO_PLUGIN_PREFIX . 'self_linking',
			'type' => 'checkbox',
			'description' => __('If you checked this option, the plugin will not replace the keywords if they appear in post/page where they are auto-linked to.','affiliate-butler-pro'),
		));
		$cmb->add_field( array(
			'name' => __( 'Auto replace keywords in excerpts', 'affiliate-butler-pro' ),
			'id'   => ABUTLER_PRO_PLUGIN_PREFIX . 'replace_in_excerpts',
			'type' => 'checkbox',
			'description' => __('If you checked this option, the plugin will not replace the keywords in excerpts (post summary).','affiliate-butler-pro'),
		));
	}

	/**
	* Validate per post limit.
	* @param	 string the option's key.
	* @return	 string the value of the key.
	* @since     2.1.1
	**/
	public function validate_per_post_limit( $value ){
		$value = intval( $value );
		return ($value === 0)?'':$value;
	}

	/**
	* Validate per post limit.
	* @param	 string the option's key.
	* @return	 string the value of the key.
	* @since     2.1.1
	**/
	public function refresh_rules( $object_id, $cmb_id, $updated ){
		if($object_id == 'abutler_settings'){
			global $wp_rewrite;
			$wp_rewrite->flush_rules( true );
			$wp_rewrite->init();
			flush_rewrite_rules( true );		
		}
	}

	/**
	* 
	* @param	 string the option's key.
	* @return	 string the value of the key.
	* @since     1.0.0
	**/
	public function get_option( $key = '' ) {
		return cmb2_get_option( $this->key, $key );
	}

	public function update_option( $key = '' ){
		return update_option($this->key, $key);
	}
	private function get_if_enabled( $default ){
		return  $this->get_option(ABUTLER_PRO_PLUGIN_PREFIX . 'enable')  ? '' : ( $default ? (string) $default : '' );
	}




	/**
	* Show `error` or `success` messages in the admin
	* @param	 string the user's message.
	* @param	 string message type (`error`,`updated`)
	* @since    1.0.0
	**/
	private function show_message( $message, $type){

		echo '<div class="' . $type . '"><p>'. $message .'</p></div>';
	}

	public function __get( $field ) {
		// Allowed fields to retrieve
		if ( in_array( $field, array( 'tabs', 'key' ), true ) ) {
			return $this->{$field};
		}

		throw new Exception( 'Invalid property: ' . $field );
	}

	/**
	* Get all registered taxonmies for the settings page.
	* @return array array of objects that contains taxonomies information.
	* @since    1.0.0
	**/
	private function get_taxonomies(){
		$taxonomies = array();
		$custom_taxonomies = get_taxonomies( 
			array( 'public'   => true,
				   '_builtin' => false),'objects');
		$builtin_taxonomies = get_taxonomies( 
			array( 'public'   => true,
				   '_builtin' => true),'objects');
		$wp_taxonomies = $builtin_taxonomies + $custom_taxonomies;
		foreach ($wp_taxonomies as $key => $value) {
			if($key != 'links_category' 
			   && $key != 'post_format'){
				$taxonomies[$value->name] = $value->labels->name;
			}
		}
		return $taxonomies;
	}

}




//$GLOBALS['abutler_settings'] = new abutler_Settings();