<?php

/**
 * The public-facing functionality of the plugin.
 *
 * @since      1.0.0
 *
 * @package    Affiliate_Butler_Pro
 * @subpackage Affiliate_Butler_Pro/public
 */

/**
 * The public-facing functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Affiliate_Butler_Pro
 * @subpackage Affiliate_Butler_Pro/public
 * @author     Waseem Senjer <waseem.senjer@gmail.com>
 */
class Affiliate_Butler_Pro_Public {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $Affiliate_Butler_Pro    The ID of this plugin.
	 */
	private $Affiliate_Butler_Pro;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	private $happened = 0;
	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $Affiliate_Butler_Pro       The name of the plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $Affiliate_Butler_Pro, $version ) {

		$this->Affiliate_Butler_Pro = $Affiliate_Butler_Pro;
		$this->version = $version;

	}

	/**
	 * Register the stylesheets for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Affiliate_Butler_Pro_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Affiliate_Butler_Pro_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		wp_enqueue_style( $this->Affiliate_Butler_Pro, plugin_dir_url( __FILE__ ) . 'css/abutler-public.css', array(), $this->version, 'all' );
		wp_enqueue_style( $this->Affiliate_Butler_Pro.'-popup', ABUTLERPRO_URL . 'public/css/popup.css', array(), $this->version, 'all' );
		wp_enqueue_style( $this->Affiliate_Butler_Pro.'-jbox', ABUTLERPRO_URL . 'public/css/jbox.css', array(), $this->version, 'all' );

	}

	/**
	 * Register the stylesheets for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Affiliate_Butler_Pro_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Affiliate_Butler_Pro_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */
		?>
			<script type="text/javascript">
				var abutler_stats_nonce = '<?php echo wp_create_nonce( 'abutler_stats_nonce' );  ?>';
				var ajaxurl = '<?php echo admin_url("admin-ajax.php"); ?>';
			</script>
		<?php
		wp_enqueue_script( $this->Affiliate_Butler_Pro.'-popup', ABUTLERPRO_URL . 'public/js/popup.js' , array( 'jquery' ), $this->version, false );
		wp_enqueue_script( $this->Affiliate_Butler_Pro.'-jbox', ABUTLERPRO_URL . 'public/js/jbox.min.js' , array( 'jquery' ), $this->version, false );
		wp_enqueue_script( $this->Affiliate_Butler_Pro, plugin_dir_url( __FILE__ ) . 'js/abutler-public.js', array( 'jquery', $this->Affiliate_Butler_Pro.'-jbox', $this->Affiliate_Butler_Pro.'-popup' ), $this->version, false );


	}

	/**
	* It will save the stats from links without slugs.
	* It will be called through AJAX request.
	*
	* @since    1.0.0
	*/
	public function store_stats(){
		check_ajax_referer( 'abutler_stats_nonce', 'security' );
		$stats = new Abutler_Stats();
		if($_POST['link_id']!='')
			$stats->save(esc_attr($_POST['link_id']), esc_attr($_POST['post_id']));
		echo '1';
		exit;
	}


	/**
	* It will save the stats from links without slugs.
	* It will be called through Action hook when the link redirected
	*
	* @since    1.0.0
	*/
	public function save_new_stat_record($link_id){

			$post_id = url_to_postid( $_SERVER['HTTP_REFERER'] );
			$stats = new Abutler_Stats();
			$stats->save($link_id,$post_id);
	}

	/**
	* add_extra_atts add the attributes of the url filled with the keyword
	* @param	 string the Link ID
	* @param	 number the Post ID
	* @return 	 string	the attributes part of the url
	* @since    1.0.0
	*/
	public function add_extra_atts($link_id, $post_id){
		$link_title = get_post_meta($link_id, ABUTLER_PRO_PLUGIN_PREFIX.'link_title',true);
		$link_type = get_post_meta($link_id, ABUTLER_PRO_PLUGIN_PREFIX.'link_type',true);
		$link_type_atts = '';
		if($link_title!=''){
			$link_title = ' title="'. htmlentities($link_title) .'" ';
		}
		if($link_type === 'external'){
			$link_type_atts = ' class="lar_link lar_link_outgoing" data-linkid="'.$link_id.'" data-postid="'.$post_id.'" ';
		}
		if($link_type === 'internal')
			$link_type_atts = ' class="lar_link" data-linkid="'.$link_id.'" data-postid="'.$post_id.'" ';
		if($link_type === 'popup')
			$link_type_atts = ' class="lar_link open-popup-link lar_link_popup" data-linkid="'.$link_id.'" data-postid="'.$post_id.'"  ';
		if($link_type === 'popup_image')
			$link_type_atts = ' class="lar_link abutler-image-link lar_link_popup" data-linkid="'.$link_id.'" data-postid="'.$post_id.'"  ';
		if($link_type === 'popup_gallery')
			$link_type_atts = ' class="lar_link abutler-gallery-link lar_link_popup" data-linkid="'.$link_id.'" data-postid="'.$post_id.'"  ';
		if($link_type === 'popup_video')
			$link_type_atts = ' class="lar_link abutlervidlink lar_link_popup" data-linkid="'.$link_id.'" data-postid="'.$post_id.'"  ';
		if($link_type === 'popup_map')
			$link_type_atts = ' class="lar_link abutlermaplink lar_link_popup" data-linkid="'.$link_id.'" data-postid="'.$post_id.'"  ';
		if($link_type === 'url_share')
			$link_type_atts = ' class="lar_link abutlertooltip" data-linkid="'.$link_id.'" ';


		return $link_type_atts.$link_title;
	}



	/**
	 * This is the main function that converts the keywords to links.
	 * It's used as (the_content, the_exerpt) filters callback.
	 * @param	 string the original content
	 * @return 	 string	the converted content.
	 * @since    1.0.0
	 */
	public function abutler_auto_replace_links( $content ){
		$abutler_global_enabled = abutler_pro()->get_option(ABUTLER_PRO_PLUGIN_PREFIX.'enable');
		if($abutler_global_enabled === false) return $content;
		$links_prefix = (abutler_pro()->get_option(ABUTLER_PRO_PLUGIN_PREFIX . 'shortlinks_prefix'))?abutler_pro()->get_option(ABUTLER_PRO_PLUGIN_PREFIX . 'shortlinks_prefix'):'go';

		// this would remove the content conflict with shortcodes
		$content = do_shortcode($content);
		// since 2.1.0
		// get limits per post
		$per_post_limit = intval(abutler_pro()->get_option(ABUTLER_PRO_PLUGIN_PREFIX.'per_post_limit'));
		if($per_post_limit === 0){ $per_post_limit = -1; }
		global $post;
		$is_disabled =  get_post_meta( $post->ID, 'abutler_disabled'  , true );

		if($is_disabled == 'on') return $content;


		$converted_taxonomies = abutler_pro()->get_option(ABUTLER_PRO_PLUGIN_PREFIX . 'link_categories');
		$taxonomies_keywords = get_terms($converted_taxonomies,'orderby=count&hide_empty=0');
		$links = get_posts('post_type=lar_link&post_status=publish&posts_per_page=-1');
        if (is_array($taxonomies_keywords)){
	        usort($taxonomies_keywords, array($this, 'sort_array_on_value'));
        }
		if(is_array($taxonomies_keywords)){
			$links = array_merge($taxonomies_keywords,$links);
		}

		foreach ($links as $link) {
			if(isset($link->ID))
				$link_meta = get_post_meta($link->ID);

			$dofollow = '';
			$link_dofollow = isset($link_meta[ABUTLER_PRO_PLUGIN_PREFIX.'do_follow'][0])?$link_meta[ABUTLER_PRO_PLUGIN_PREFIX.'do_follow'][0]:'';
			if($link_dofollow !== 'on'){
				$dofollow = 'rel="nofollow"';
			}
			$link_type = (isset($link_meta[ABUTLER_PRO_PLUGIN_PREFIX.'link_type'][0]))?$link_meta[ABUTLER_PRO_PLUGIN_PREFIX.'link_type'][0]:'';
			if($link_type == 'external'){
				if ( get_option('permalink_structure') != '' ) {
					$url = (isset($link_meta[ABUTLER_PRO_PLUGIN_PREFIX.'slug'][0]) && $link_meta[ABUTLER_PRO_PLUGIN_PREFIX.'slug'][0]!= '')? site_url().'/'. $links_prefix .'/'.$link_meta[ABUTLER_PRO_PLUGIN_PREFIX.'slug'][0] : $link_meta[ABUTLER_PRO_PLUGIN_PREFIX.'url'][0];
				}else{
					$url = (isset($link_meta[ABUTLER_PRO_PLUGIN_PREFIX.'slug']) && $link_meta[ABUTLER_PRO_PLUGIN_PREFIX.'slug'][0] != '')? site_url().'/index.php?'. $links_prefix .'='.$link_meta[ABUTLER_PRO_PLUGIN_PREFIX.'slug'][0] : $link_meta[ABUTLER_PRO_PLUGIN_PREFIX.'url'][0];

				}
			}elseif($link_type == 'internal'){ // if internal link
					$url = get_permalink($link_meta[ABUTLER_PRO_PLUGIN_PREFIX.'internal_url'][0]);
			}elseif($link_type == 'popup'){ // if internal link
					$url = '#abutler_popup_'. $link->ID;
			}elseif($link_type == 'popup_image'){
					$url = isset($link_meta[ABUTLER_PRO_PLUGIN_PREFIX.'popup_image'][0])?$link_meta[ABUTLER_PRO_PLUGIN_PREFIX.'popup_image'][0]:'';
			}elseif($link_type == 'popup_gallery'){
					$url = 'javascript:void(0)';
			}elseif($link_type == 'popup_video'){
					$url = isset($link_meta[ABUTLER_PRO_PLUGIN_PREFIX.'popup_videourl'][0])?$link_meta[ABUTLER_PRO_PLUGIN_PREFIX.'popup_videourl'][0]:'';
			}elseif($link_type == 'popup_map'){
					$url = isset($link_meta[ABUTLER_PRO_PLUGIN_PREFIX.'popup_mapurl'][0])?$link_meta[ABUTLER_PRO_PLUGIN_PREFIX.'popup_mapurl'][0]:'';
					$url_segments = explode('/', $url);
					$url = 'https://maps.google.com/maps?q='. $url_segments[5] .'&hl=en';
			}elseif($link_type == 'url_share'){
					$url = "javascript:void(0)";
			}



			if(isset($link->term_id)){
				$keywords = array();
				$keywords[] = $link->name;
				$url = get_term_link(intval($link->term_id), $link->taxonomy);
			}else{
				if(isset($link_meta[ABUTLER_PRO_PLUGIN_PREFIX.'keywords'][0]))
					$keywords = unserialize( $link_meta[ABUTLER_PRO_PLUGIN_PREFIX.'keywords'][0] );
				else
					$keywords = false;
			}
			// if the keywords not really inserted
			if($keywords === false) continue;
			libxml_use_internal_errors(true);
			$doc = new DOMDocument();
			@$doc->loadHTML('<?xml encoding="UTF-8">'.$content);
			$doc->encoding = 'UTF-8';

			foreach($keywords as $keyword){

				$keyword = html_entity_decode(stripslashes(wptexturize($keyword)));
				if(abutler_pro()->get_option(ABUTLER_PRO_PLUGIN_PREFIX . 'self_linking') != false){
					if($url == get_permalink($post->ID) ) continue;
				}
				if(!isset($link->ID)){
					$extra_attrs = ' class="abutler-taxonomy-link" ';
				}else{
					$extra_attrs = apply_filters('abutler_add_extra_atts',$link->ID, $post->ID);
				}
				$open_in  = isset($link_meta[ABUTLER_PRO_PLUGIN_PREFIX.'open_in'][0])?'target="'.$link_meta[ABUTLER_PRO_PLUGIN_PREFIX.'open_in'][0].'"':'';
				$final_url = ' <a href="'.$url.'" '.$extra_attrs.' '.$dofollow.' '. $open_in .' >${1}</a>';


				// sensitivity modifier
				$i = (isset($link_meta[ABUTLER_PRO_PLUGIN_PREFIX.'is_sensitive'][0]) && $link_meta[ABUTLER_PRO_PLUGIN_PREFIX.'is_sensitive'][0] === 'on')?'':'i';
				/** @var DOMDocument $changed */
				$changed =  $this->showDOMNode($doc,$keyword,$final_url,$i, $per_post_limit);
				$this->happened = 0;

				$mock = new DOMDocument;
			    $body = $changed->getElementsByTagName('body')->item(0);
				if(isset($body->childNodes)){
				    /** @var DOMNode $child */
					foreach ($body->childNodes as $child){
					    $mock->appendChild($mock->importNode($child, true));
					}
				}

				$content = $this->handle_pre_html_tag(html_entity_decode(htmlspecialchars_decode($mock->SaveHTML())));
				$doc = new DOMDocument();
				@$doc->loadHTML('<?xml encoding="UTF-8">'.$content);
				$doc->encoding = 'UTF-8';
				// @since 2.1 adding the popup div
				if($link_type == 'popup'){
					$content .= '<div id="abutler_popup_'. $link->ID.'" class="white-popup mfp-hide">';
					$content .= (isset($link_meta[ABUTLER_PRO_PLUGIN_PREFIX.'popup_content'][0]))?$link_meta[ABUTLER_PRO_PLUGIN_PREFIX.'popup_content'][0]:'';
					$content .= '</div>';
				}elseif($link_type == 'popup_gallery'){
					$content .= '<div class="abutler-gallery" id="abutler_gallery_'.$link->ID.'">';
					$images  = unserialize($link_meta[ABUTLER_PRO_PLUGIN_PREFIX.'popup_gallery'][0]);
					foreach($images as $image){

						$content .= '<a href="'. $image .'"></a>';
					}
					$content .= '</div>';
				}elseif($link_type == 'url_share'){
					$shared_url = urlencode((isset($link_meta[ABUTLER_PRO_PLUGIN_PREFIX.'shared_url'][0]))?$link_meta[ABUTLER_PRO_PLUGIN_PREFIX.'shared_url'][0]:'');
					$shared_text = urlencode((isset($link_meta[ABUTLER_PRO_PLUGIN_PREFIX.'shared_text'][0]))?$link_meta[ABUTLER_PRO_PLUGIN_PREFIX.'shared_text'][0]:'');
					$content .= '<div id="abutlershare_for_'.$link->ID.'" style="display:none;">';
					$content .= '<a class="abutlersharepopup" href="http://www.facebook.com/sharer/sharer.php?u='.$shared_url.'&t='. $shared_text .'"><img class="abutler_social_share_icon" src="'.ABUTLERPRO_URL.'public/images/share/style01/facebook.png" /></a>';
					$content .= '<a class="abutlersharepopup" href="https://twitter.com/intent/tweet?url='.$shared_url.'&text='.$shared_text.'"><img class="abutler_social_share_icon" src="'.ABUTLERPRO_URL.'public/images/share/style01/twitter.png" /></a>';
					$content .= '<a class="abutlersharepopup" href="https://www.linkedin.com/shareArticle?mini=true&url='.$shared_url.'&title='.$shared_text.'&summary=&source="><img class="abutler_social_share_icon" src="'.ABUTLERPRO_URL.'public/images/share/style01/linkedin.png" /></a>';
					$content .= '<a class="abutlersharepopup" href="http://plus.google.com/share?url='.$shared_url.'"><img class="abutler_social_share_icon" src="'.ABUTLERPRO_URL.'public/images/share/style01/gplus.png" /></a>';
					$content .= '<a class="abutlersharepopup" href="https://www.tumblr.com/widgets/share/tool?canonicalUrl='.$shared_url.'&url='.$shared_url.'&posttype=link&title='.$shared_text.'&caption='.$shared_text.'&content='.$shared_url.'"><img class="abutler_social_share_icon" src="'.ABUTLERPRO_URL.'public/images/share/style01/tumblr.png" /></a>';
					$content .= '<a class="abutlersharepopup" href="http://www.reddit.com/submit?url='.$shared_url.'&title='.$shared_text.'"><img class="abutler_social_share_icon" src="'.ABUTLERPRO_URL.'public/images/share/style01/reddit.png" /></a>';
					$content .= '</div>';
				}
			}


		}

		// Replace Content Filter
		$content = apply_filters('abutler_replace_content', $content);

		return $content;
	}

	/**
	 * @param $content
	 *
	 * @return string
	 */
	public function abutler_auto_replace_links_excerpt( $content ){
		if(abutler_pro()->get_option(ABUTLER_PRO_PLUGIN_PREFIX . 'replace_in_excerpts') === 'on'){
		    return $this->abutler_auto_replace_links($content);
		}
		return $content;
    }

	/**
	 * Helper function that is used to get text nodes from the DomNode and replace the keywords
	 * It's used in $this -> abutler_auto_replace_links()
	 *
	 * @param     DOMNode $domNode current node
	 * @param     string $word The keyword that should be replaced.
	 * @param     string $replacement The replacement string, it's the url in our context.
	 * @param $case_sensitive
	 * @param int $per_post_limit
	 *
	 * @return DOMNode The replaced Node.
	 * @since    1.0.0
	 */
	public function showDOMNode(DOMNode $domNode,$word,$replacement,$case_sensitive, $per_post_limit = -1) {
		$excluded_html_tags = abutler_pro()->get_option(ABUTLER_PRO_PLUGIN_PREFIX.'excluded_html_tags');
		$excluded_html_tags = explode(',', $excluded_html_tags);
	    /** @var DOMNode $node */
		foreach ($domNode->childNodes as $node)
	    {
	    	// Pass the nodes which already linked.
	        if($node->nodeName == 'a') continue;
	        if( in_array($node->nodeName, $excluded_html_tags) ) continue;

	        if($node->nodeName == '#text'){
	        	$occurencies = $this->count_occurencies ( strtoupper( $node->nodeValue ), strtoupper( $word ),$case_sensitive );
	        	$limit = ($per_post_limit != -1)?$per_post_limit - $this->happened:$per_post_limit;
	        	if($this->happened < $per_post_limit || $per_post_limit === -1){
			        $replacement = $this->clean_replacement($replacement);
			        $replacement_pattern = $this->get_replacement_pattern($word, $case_sensitive);
			        $node->nodeValue =  @preg_replace($replacement_pattern, $replacement, $node->nodeValue, $limit);
	        	}
	        	if($occurencies > 0){
	        		$this->happened  = $this->happened + $occurencies;
	        	}
                if($per_post_limit !== -1){
	                if($this->happened >= $per_post_limit){
		                return $domNode;
	                }
                }

	        }
	        if($node->hasChildNodes()) {
	            $this->showDOMNode($node,$word,$replacement,$case_sensitive, $per_post_limit);
	        }
	    }
	    return $domNode;
	}

	/**
	 * Calculate the occurencies of a word in a string.
	 *
	 * @param $str
	 * @param $word
	 * @param $case_sensitive
	 *
	 * @since    1.1.0
	 * @return int
	 */
	private function count_occurencies($str, $word, $case_sensitive = '') {
	    return preg_match_all($this->get_replacement_pattern($word, $case_sensitive), $str);
 	}

	/**
	 * This method adds a new rewrite rules for external links with slugs.
	 * @param	 array  The original rules.
	 * @return	 array The new rules.
	 * @since    1.0.0
	 */
	public function abutler_setup_rewrite_rules($rules)
	{
		if(function_exists('cmb2_get_option')){
			$links_prefix = (abutler_pro()->get_option(ABUTLER_PRO_PLUGIN_PREFIX . 'shortlinks_prefix'))?abutler_pro()->get_option(ABUTLER_PRO_PLUGIN_PREFIX . 'shortlinks_prefix'):'go';
	    	$newrules['^'. $links_prefix .'/([^/]*)/?$'] = 'index.php?'.$links_prefix.'=$matches[1]';
	    	$newrules['^index.php?'. $links_prefix .'=([^/]*)?$'] = 'index.php?'. $links_prefix .'=$matches[1]';
	    	return $newrules + $rules;
	    }
	    return $rules;
	}

	/**
	 * This method adds a new query var [go]
	 * @TODO Should be enhanced by removing the [go] prefix from urls with slugs.
	 *
	 * @param	 array  The original vars.
	 * @return	 array The new vars with [go] added.
	 * @since    1.0.0
	 */
	public function add_go_variable($vars)
	{
		if(function_exists('cmb2_get_option')){
			$links_prefix = (abutler_pro()->get_option(ABUTLER_PRO_PLUGIN_PREFIX . 'shortlinks_prefix'))?abutler_pro()->get_option(ABUTLER_PRO_PLUGIN_PREFIX . 'shortlinks_prefix'):'go';
	 	    array_push($vars, $links_prefix);
	    }
	    return $vars;
	}


	/**
	* This method is responsible for redirecting slugged links to its original urls.
	* Also, it, it will save a new stats record.
	*
	* @since    1.0.0
	*/
	function abutler_redirect(){
		global $wp_query;
		$links_prefix = (abutler_pro()->get_option(ABUTLER_PRO_PLUGIN_PREFIX . 'shortlinks_prefix'))?abutler_pro()->get_option(ABUTLER_PRO_PLUGIN_PREFIX . 'shortlinks_prefix'):'go';

		if(isset($wp_query->query_vars[$links_prefix])){
			global $wpdb;
			$link = get_posts('post_type=lar_link&meta_key='.ABUTLER_PRO_PLUGIN_PREFIX.'slug&meta_value='.$wp_query->query_vars[$links_prefix]);
			$link_url = get_post_meta($link[0]->ID, ABUTLER_PRO_PLUGIN_PREFIX.'url',true);

			if(!is_null($link_url)){
				do_action('lar_link_redirected', $link[0]->ID);
				wp_redirect($link_url);
				exit;
			}

		}
	}
	/**
	* Sort array based on length of the values.
	*
	* @since    2.1.3
	*/
	public function sort_array_on_value($a,$b){
        return strlen($b->name) - strlen($a->name);
	}

	/**
	 * @param $replacement
	 *
	 * @return string
	 */
	private function clean_replacement( $replacement ) {
		$concatenated_words_langs = array('ja', 'zh_CN', 'zh_HK', 'zh_TW');
		if(in_array(get_locale(), $concatenated_words_langs)){
	        $replacement = trim($replacement);
        }
	    return $replacement;
	}

	/**
	 * @param $word
	 * @param $case_sensitive
	 *
	 * @return string
	 */
	private function get_replacement_pattern( $word, $case_sensitive ) {
	    $word = preg_quote($word, '/');
	    $concatenated_words_langs = array('ja', 'zh_CN', 'zh_HK', 'zh_TW');
		if(in_array(get_locale(), $concatenated_words_langs)){
			return '/('.($word).')(?![\w-])/'.$case_sensitive.'u';
		}

	    return '/\b('.($word).')(?![\w-])/'.$case_sensitive.'u';
	}


	private function handle_pre_html_tag( $content ) {
		$pattern = '/<pre(.*?)>(.*?)<\/pre>/sm';
		preg_match_all($pattern, $content, $matches, PREG_OFFSET_CAPTURE);
		if (!isset($matches[2])) {
		    return $content;
        }
		if ( is_array($matches[2]) ){
			foreach ( $matches[2] as $match ) {
				$with_html_entities = htmlentities($match[0]);
				$content = str_replace($match[0], $with_html_entities, $content);
			}
        }

		return $content;
	}
}
